"""
Built-in validate comparators.
"""

import re
from apicase.common.schema import JSONAssertSchema


def _cast_to_int(expect_value):
    try:
        return int(expect_value)
    except Exception:
        raise AssertionError("%r can't cast to int" % str(expect_value))


# from httprunner.compat import basestring, builtin_str, integer_types
def equals(check_value, expect_value):
    assert check_value == expect_value


def less_than(check_value, expect_value):
    assert check_value < expect_value


def less_than_or_equals(check_value, expect_value):
    assert check_value <= expect_value


def greater_than(check_value, expect_value):
    assert check_value > expect_value


def greater_than_or_equals(check_value, expect_value):
    assert check_value >= expect_value


def not_equals(check_value, expect_value):
    assert check_value != expect_value


# def string_equals(check_value, expect_value):
#     assert builtin_str(check_value) == builtin_str(expect_value)


# def length_equals(check_value, expect_value):
#     assert isinstance(expect_value, integer_types)
#     expect_len = _cast_to_int(expect_value)
#     assert len(check_value) == expect_len


def length_greater_than(check_value, expect_value):
    assert isinstance(expect_value, int)
    expect_len = _cast_to_int(expect_value)
    assert len(str(check_value)) > expect_len

length_greater_than('10008', 1)
# def length_greater_than_or_equals(check_value, expect_value):
#     assert isinstance(expect_value, integer_types)
#     expect_len = _cast_to_int(expect_value)
#     assert len(check_value) >= expect_len
#
#
# def length_less_than(check_value, expect_value):
#     assert isinstance(expect_value, integer_types)
#     expect_len = _cast_to_int(expect_value)
#     assert len(check_value) < expect_len
#
#
# def length_less_than_or_equals(check_value, expect_value):
#     assert isinstance(expect_value, integer_types)
#     expect_len = _cast_to_int(expect_value)
#     assert len(check_value) <= expect_len
#
#
# def contains(check_value, expect_value):
#     assert isinstance(check_value, (list, tuple, dict, basestring))
#     assert expect_value in check_value
#
#
# def contained_by(check_value, expect_value):
#     assert isinstance(expect_value, (list, tuple, dict, basestring))
#     assert check_value in expect_value
#
#
# def type_match(check_value, expect_value):
#     def get_type(name):
#         if isinstance(name, type):
#             return name
#         elif isinstance(name, basestring):
#             try:
#                 return __builtins__[name]
#             except KeyError:
#                 raise ValueError(name)
#         else:
#             raise ValueError(name)
#
#     assert isinstance(check_value, get_type(expect_value))
#
#
# def regex_match(check_value, expect_value):
#     assert isinstance(expect_value, basestring)
#     assert isinstance(check_value, basestring)
#     assert re.match(expect_value, check_value)
#
#
# def startswith(check_value, expect_value):
#     assert builtin_str(check_value).startswith(builtin_str(expect_value))
#
#
# def endswith(check_value, expect_value):
#     assert builtin_str(check_value).endswith(builtin_str(expect_value))
#
#
# def _cast_to_int(expect_value):
#     try:
#         return int(expect_value)
#     except Exception:
#         raise AssertionError("%r can't cast to int" % str(expect_value))


class AssertJson:
    """
    工具类
    """

    @staticmethod
    def equals(jmespath, expect_value):
        """
        相等
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=equals)

    @staticmethod
    def less_than(jmespath, expect_value):
        """
        小于
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=less_than)

    @staticmethod
    def less_than_or_equals(jmespath, expect_value):
        """
        小于或等于
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=less_than_or_equals)

    @staticmethod
    def greater_than(jmespath, expect_value):
        """
        大于
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=greater_than)

    @staticmethod
    def greater_than_or_equals(jmespath, expect_value):
        """
        大于或等于
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=greater_than_or_equals)

    @staticmethod
    def not_equals(jmespath, expect_value):
        """
        不等于
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=not_equals)

    @staticmethod
    def length_greater_than(jmespath, expect_value: int):
        """
        长度大于
        """
        return JSONAssertSchema(jmespath=jmespath, expectations=expect_value, comparator=length_greater_than)


assertJSON = AssertJson()



import json
import allure


class AllureAttach(object):

    @classmethod
    def api_record(cls, response):
        name = "{0} {1}".format(response.request.method, response.request.url)
        req_headers = json.dumps(dict(response.request.headers), ensure_ascii=False)
        res_headers = json.dumps(dict(response.headers), ensure_ascii=False)
        req_body = response.request.body
        if type(req_body) == bytes:
            try:
                req_body = req_body.decode()
            except UnicodeDecodeError:
                req_body = 'form-data参数不做打印'
        data_template = {
            'request': {
                'body': req_body,
                'headers': req_headers,
            },
            'response': {
                'status_code': response.status_code or None,
                'body': response.text,
                'headers': res_headers,
                # 'cookies': response.cookies or None,
                'encoding': response.encoding,
                'reason': response.reason,
                'ok': response.ok,
                'response_time_s': response.elapsed.total_seconds(),
            }
        }
        data_template = json.dumps(data_template, indent=1, separators=(',          ', ': '), ensure_ascii=False)
        allure.attach(body=data_template, name=name, attachment_type=allure.attachment_type.JSON)

    @classmethod
    def sql_record(cls, sql_info: dict):
        crud = sql_info['crud']
        url = sql_info['url']
        name = None
        for i in crud:
            if crud[i]:
                name = str(i) + ' ' + str(url)
                break
        data = str(sql_info['sql']) + '\n' + str(sql_info['parameters'])
        allure.attach(body=data, name=name, attachment_type=allure.attachment_type.TEXT)

    @classmethod
    def schema_assert_record(cls, assert_json: dict, schema: dict) -> None:
        """
        TODO(GuoWenHe): 待扩展数组模式
        """
        assert_json = json.dumps(assert_json, ensure_ascii=False)
        schema = json.dumps(schema, ensure_ascii=False)
        name = 'Assert api_json->schema'
        data = 'assert_data:\n' + str(assert_json) + '\nassert_schema\n' + str(schema)
        allure.attach(body=data, name=name, attachment_type=allure.attachment_type.TEXT)

    @classmethod
    def diff_record(cls, json_data) -> None:
        name = 'Assert Diff'
        data_template = json.dumps(json_data, indent=1, separators=(',  ', ': '), ensure_ascii=False)
        allure.attach(body=data_template, name=name, attachment_type=allure.attachment_type.JSON)



import argparse
from apicase import __description__, __version__


def main():
    """ API test: parse command line options and run commands.
    """
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument(
        "-V", "--version", dest="version", action="store_true", help="显示版本"
    )
    #
    # subparsers = parser.add_subparsers(help="sub-command help")
    # sub_parser_run = init_parser_run(subparsers)
    # sub_parser_scaffold = init_parser_scaffold(subparsers)
    # sub_parser_har2case = init_har2case_parser(subparsers)
    # sub_parser_make = init_make_parser(subparsers)
    #
    # if len(sys.argv) == 1:
    #     # httprunner
    #     parser.print_help()
    #     sys.exit(0)
    # elif len(sys.argv) == 2:
    #     # print help for sub-commands
    #     if sys.argv[1] in ["-V", "--version"]:
    #         # httprunner -V
    #         print(f"{__version__}")
    #     elif sys.argv[1] in ["-h", "--help"]:
    #         # httprunner -h
    #         parser.print_help()
    #     elif sys.argv[1] == "startproject":
    #         # httprunner startproject
    #         sub_parser_scaffold.print_help()
    #     elif sys.argv[1] == "har2case":
    #         # httprunner har2case
    #         sub_parser_har2case.print_help()
    #     elif sys.argv[1] == "run":
    #         # httprunner run
    #         pytest.main(["-h"])
    #     elif sys.argv[1] == "make":
    #         # httprunner make
    #         sub_parser_make.print_help()
    #     sys.exit(0)
    # elif (
    #         len(sys.argv) == 3 and sys.argv[1] == "run" and sys.argv[2] in ["-h", "--help"]
    # ):
    #     # httprunner run -h
    #     pytest.main(["-h"])
    #     sys.exit(0)
    #
    # extra_args = []
    # if len(sys.argv) >= 2 and sys.argv[1] in ["run", "locusts"]:
    #     args, extra_args = parser.parse_known_args()
    # else:
    #     args = parser.parse_args()
    #
    # if args.version:
    #     print(f"{__version__}")
    #     sys.exit(0)
    #
    # if sys.argv[1] == "run":
    #     sys.exit(main_run(extra_args))
    # elif sys.argv[1] == "startproject":
    #     main_scaffold(args)
    # elif sys.argv[1] == "har2case":
    #     main_har2case(args)
    # elif sys.argv[1] == "make":
    #     main_make(args.testcase_path)



from typing import Union, List, Any
import jmespath
from apicase.common.exception import JsonAssertError
from apicase.common.enumeration import ComparatorEnum, CheckResultEnum
from apicase.common.schema import JSONAssertSchema


class Asserter(object):

    def _assert(self):
        pass


class JSONAsserter(Asserter):
    """
    json 断言器
    """
    assert_list = List[JSONAssertSchema]
    assert_results_list = List

    def __init__(self, assert_list: List[JSONAssertSchema], response_json: Union[dict, list],
                 equal_type: bool = True, ):
        self.assert_list = assert_list
        self.equal_type = equal_type
        # self.init_data()
        self.json_assert(response_json)

    # def init_data(self):
    #     """
    #     将断言列表转换为JSONAssertSchema 格式
    #     :return:
    #     """
    #     assert_list = []
    #     for i in self.a_list:
    #         assert_list.append(
    #             JSONAssertSchema(jmespath=i[0], comparator=i[1], expectations=i[2])
    #         )
    #     self.assert_list = assert_list

    def json_assert(self, response_json: Union[dict, list]):
        """
        json断言
        :param response_json: 请求响应json
        :return:
        """
        assert_results_list = []
        for i in self.assert_list:
            i.value = jmespath.search(i.jmespath, response_json)
            try:
                i.comparator(i.value, i.expectations)
            except AssertionError:
                i.check = CheckResultEnum.FAIL
            else:
                i.check = CheckResultEnum.PASS
            assert_results_list.append(i)
        self.assert_results_list = assert_results_list
        # self._assert()

    # def _assert(self):
    #     error_list = []
    #     for i in self.assert_results_list:
    #         if not i.check:
    #             error_list.append(i)
    #     error_str = ''
    # for i in error_list:
    #     assert_example = i[2]
    #     value = str(assert_example.value)
    #     assert_value = str(i[0])
    #     comparator = assert_example.comparator
    #     expression = assert_example.expression
    #     error_str += '\n\t JSON.{expression} {comparator} {value} \t' \
    #                  'But: {assert_value} not {comparator} {value}'.format(expression=expression,
    #                                                                        comparator=comparator.value,
    #                                                                        value=value,
    #                                                                        assert_value=assert_value,
    #                                                                        )
    # raise JsonAssertError('JSON 断言失败' + error_str)
    # print('断言通过', str(self.assert_results_list))


class ResponseAsserter(Asserter):
    pass


class CustomAsserter(Asserter):

    def __init__(self, a_list, response):
        self.a_list = a_list
        self.response = response

    def _assert(self):
        for i in self.a_list:
            i(self.response)



__version__ = '0.1.2'
__description__ = '接口自动化用例、测试的一站式解决方案。'

import requests
from apicase.schema import BaseModel
from apicase.client.router import APIRouter
from apicase.client.reuquest import APIRequest
from apicase.client.reuquest import session
from apicase.assertion.comparators import assertJSON
from apicase.common.enumeration import (
    BodyTypeEnum,
)
from apicase.common.schema import (
    RequestSchema,
)
from apicase.common.exception import (
    JsonAssertError
)
from apicase.common.logger import log
from apicase.setting import Settings, BaseSettingSchema

bt = BodyTypeEnum



import os
import yaml
from typing import Any, Dict
from pydantic import Extra, BaseSettings as Base


class BaseSettingSchema(Base):
    _yaml_path: str

    @classmethod
    def yaml_config_settings_source(cls, setting: Base) -> Dict[str, Any]:
        """
        A simple settings source that loads variables from a JSON file
        at the project's root.
        """
        encoding = setting.__config__.env_file_encoding
        with open(cls._yaml_path, encoding=encoding) as f:
            return yaml.load(f, Loader=yaml.FullLoader)

    class Config:
        env_file_encoding = 'utf-8'
        extra: Extra = Extra.allow

        @classmethod
        def customise_sources(
                cls,
                init_settings,
                env_settings,
                file_secret_settings,
        ):
            """
            None
            :param init_settings:
            :param env_settings:
            :param file_secret_settings:
            :return:
            """
            return (
                init_settings,
                BaseSettingSchema.yaml_config_settings_source,
                env_settings,
                file_secret_settings,
            )


class Settings(object):
    """
    :param
    """

    def __init__(self, env: str, env_suffix: str, env_path: str, setting_class):
        BaseSettingSchema._yaml_path = os.path.join(env_path, env + env_suffix)
        self.setting_class = setting_class

    def get_setting(self):
        """

        :return:
        """
        return self.setting_class()






import logging
from typing import Union, List, Any
import jmespath
import loguru

from apicase.common.schema import JSONAssertSchema
from apicase.assertion.assertion import (
    JSONAsserter, CustomAsserter
)
from apicase.report.attach import AllureAttach
from apicase.common.enumeration import CheckResultEnum
from apicase.common.exception import JsonAssertError


class APIResponse(object):
    """
    http响应数据
    """
    response_json: Union[dict, list] = None

    def __init__(self, response,
                 default_json_extraction: str = None,
                 default_json_asserter_list: List[JSONAssertSchema] = None,
                 json_asserter_list: List[JSONAssertSchema] = None,
                 default_custom_asserter_list: List = None,
                 custom_asserter_list: List = None,
                 description: str = None,
                 path: str = None
                 ):
        self.validators = None
        self.extraction_json = None
        self.request = response.request
        self.response = response
        self.status_code = response.status_code
        self.headers = response.headers
        self.json = response.json
        self.text = response.text
        self.default_json_extraction = default_json_extraction
        self.default_json_asserter_list = default_json_asserter_list
        self.json_asserter_list = json_asserter_list
        self.default_custom_asserter_list = default_custom_asserter_list
        self.custom_asserter_list = custom_asserter_list

        self.api_assert()

        AllureAttach.record_http(
            response=self.response, validators=self.validators, description=description, path=path
        )
        self.raise_failure()

    def raise_failure(self):
        error_str = ''

        for i in self.validators:
            if i.check == CheckResultEnum.FAIL:
                error_str += '\n\t检查:{},比较器:{},期望:{},实际:{}'.format(i.jmespath, i.comparator, i.expectations, i.value)
        if error_str:
            raise JsonAssertError('JSON 断言失败' + error_str)

    def get_extraction_json(self) -> dict:
        """
        获取提取后的json数据
        :return:
        """
        self._init_json()
        return self.extraction_json

    def _init_json(self):
        if not self.response_json:
            self.response_json = self.json()
        if not self.default_json_extraction:
            self.extraction_json = self.response_json
        else:
            extraction_json = jmespath.search(expression=self.default_json_extraction, data=self.response_json)
            self.extraction_json = extraction_json

    def jmespath(self, *expressions):
        """
        jsonpath提取器
        :param expressions: 单个或多个要提取的jsonpath字符串
        :return: 返回单个或多个提取的数据，可使用解包接收
                var1,var2,var3 = api.send().jmespath('data.id', 'data.name')
        """
        self._init_json()
        expressions = list(expressions)
        extraction_list = []
        # if not self.default_json_extraction:
        #     self.extraction_json = self.response_json
        # else:
        #     extraction_json = jmespath.search(expression=self.default_json_extraction, data=self.response_json)
        #     self.extraction_json = extraction_json
        for i in range(len(expressions)):
            expression = expressions[i]
            extraction_list.append(jmespath.search(expression=expression, data=self.extraction_json))
        if len(extraction_list) == 1:
            return extraction_list[0]
        return tuple(extraction_list)

    def jsonpath(self, expression):
        pass

    def api_assert(self):
        """

        :param self:
        """
        self._init_json()
        validators = []
        if self.default_json_asserter_list:
            validators += JSONAsserter(self.default_json_asserter_list, self.response_json).assert_results_list
        if self.json_asserter_list:
            validators += JSONAsserter(self.json_asserter_list, self.extraction_json).assert_results_list
        if self.default_custom_asserter_list:
            CustomAsserter(self.default_custom_asserter_list, self)
        if self.custom_asserter_list:
            CustomAsserter(self.custom_asserter_list, self)
        self.validators = validators



from typing import Any, List

from apicase.client.reuquest import APIRequest
from apicase.common.enumeration import BodyTypeEnum
from apicase.common.schema import JSONAssertSchema


class APIRouter:

    def __init__(self,
                 base_url: str,
                 intermediate_route: str = None,
                 pre: list = None,
                 after: list = None,
                 default_json_extraction: str = None,
                 default_json_asserter_list: List = None,
                 default_custom_asserter_list: List = None,
                 ):
        """
        APIRouter
        :param base_url: 基础路由
        :param intermediate_route: 服务路由
        :param pre: 全局前置处理器
        :param after: 全局后置处理器
        :param default_json_extraction: 公共JSON提取器
        :param default_json_asserter_list: 全局默认断言列表
        :param default_custom_asserter_list: 全局默认自定义处理器
        """
        self.base_url = base_url
        self.intermediate_route = intermediate_route
        if intermediate_route:
            self.base_url = self.base_url + self.intermediate_route
        self.pro = pre
        self.after = after
        self.default_json_extraction = default_json_extraction
        self.default_json_asserter_list = default_json_asserter_list
        self.default_custom_asserter_list = default_custom_asserter_list

    def api_route(self,
                  method: str,
                  path: str,
                  body_type: BodyTypeEnum,
                  pre: list = None,
                  after: list = None,
                  other: dict = None,
                  description: str = None,
                  default_json_asserter_list: list = None,
                  ) -> APIRequest:
        url = self.base_url + path
        if pre is None:
            pre = list()
        if after is None:
            after = list()
        if self.pro:
            pre = self.pro + pre
        if self.after:
            after = self.after + after
        json_asserter_list = self.default_json_asserter_list
        if default_json_asserter_list:
            json_asserter_list = default_json_asserter_list
        api = APIRequest(
            url=url,
            method=method,
            body_type=body_type,
            pre=pre,
            after=after,
            other=other,
            path=self.intermediate_route + path,
            description=description,
            default_json_extraction=self.default_json_extraction,
            default_json_asserter_list=json_asserter_list,
            default_custom_asserter_list=self.default_custom_asserter_list,
        )
        return api

    def post(self,
             path: str,
             request_model: Any = None,
             query_params_model: Any = None,
             body_type: BodyTypeEnum = BodyTypeEnum.NULL,
             pre: list = None,
             after: list = None,
             other: dict = None,
             description: str = None,
             default_json_asserter_list: list = None
             ) -> APIRequest:
        """
        request_model 因达不到类型提示的效果，仅做注释关联使用
        """
        if request_model:
            body_type = BodyTypeEnum.JSON
        return self.api_route(
            method='post',
            path=path,
            body_type=body_type,
            pre=pre,
            after=after,
            other=other,
            description=description,
            default_json_asserter_list=default_json_asserter_list,
        )

    def get(self,
            path: str,
            query_params_model: Any = None,
            pre: list = None,
            after: list = None,
            other: dict = None,
            description: str = None,
            default_json_asserter_list: list = None
            ) -> APIRequest:
        return self.api_route(
            method='get',
            path=path,
            body_type=BodyTypeEnum.NULL,
            pre=pre,
            after=after,
            other=other,
            description=description,
            default_json_asserter_list=default_json_asserter_list,
        )



import io
import os
from jinja2 import Template


def gen_html_report(summary):
    """ render html report with specified report name and template

    Args:
        summary (dict): test result summary data
        report_template (str): specify html report template path, template should be in Jinja2 format.
        report_dir (str): specify html report save directory
        report_file (str): specify html report file path, this has higher priority than specifying report dir.
    """

    report_template = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        "template.html"
    )

    # start_at_timestamp = summary["time"]["start_at"]
    # summary["time"]["start_datetime"] = start_at_timestamp

    with io.open(report_template, "r", encoding='utf-8') as fp_r:
        template_content = fp_r.read()
        rendered_content = Template(
            template_content,
            extensions=["jinja2.ext.loopcontrols"]
        ).render(summary)
        return rendered_content






# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['apicase',
 'apicase.assertion',
 'apicase.client',
 'apicase.common',
 'apicase.report',
 'apicase.report.html',
 'apicase.script']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'PyYAML>=6.0,<7.0',
 'allure-pytest>=2.12.0,<3.0.0',
 'jmespath>=1.0.1,<2.0.0',
 'loguru>=0.6.0,<0.7.0',
 'pydantic[dotenv]>=1.10.5,<2.0.0',
 'pytest-assume>=2.4.3,<3.0.0',
 'pytest-sugar>=0.9.6,<0.10.0',
 'requests>=2.28.1,<3.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['apicase = apicase.script.cli:main']}

setup_kwargs = {
    'name': 'apicase',
    'version': '0.1.4',
    'description': '',
    'long_description': None,
    'author': 'guowenhe',
    'author_email': '18538570410@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9.0,<4.0.0',
}


setup(**setup_kwargs)



import warnings
from enum import Enum
from datetime import datetime
from typing import (
    List, Dict, Set, Tuple, Optional, Union
)
from pydantic import (
    BaseModel as Base, BaseSettings, PyObject, RedisDsn, PostgresDsn, AmqpDsn, Field,
)


def to_camel(string: str) -> str:
    """
    蟒蛇字符转换为小驼峰
    :param string:
    :return:
    """
    s = ''.join(word.capitalize() for word in string.split('_'))
    return s[0].lower() + s[1:]


class BaseModel(Base):
    class Config:
        arbitrary_types_allowed = True
        allow_population_by_field_name = False
        # alias_generator = to_camel  有bug，不能用
        smart_union = False

    def dict(
            self,
            *,
            include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
            exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
            by_alias: bool = False,
            skip_defaults: Optional[bool] = None,
            exclude_unset: bool = False,
            exclude_defaults: bool = False,
            exclude_none: bool = True,
    ) -> 'DictStrAny':
        """
        Generate a dictionary representation of the model, optionally specifying which fields to include or exclude.

        重写覆盖父类转字典方法，差异内容如下：
            exclude_none（排除等于none的字段） 修改为默认等于True，大多数接口传null代表修改为空，实际不修改某一内容应不传递参数
        """
        if skip_defaults is not None:
            warnings.warn(
                f'{self.__class__.__name__}.dict(): "skip_defaults" is deprecated and replaced by "exclude_unset"',
                DeprecationWarning,
            )
            exclude_unset = skip_defaults

        return dict(
            self._iter(
                to_dict=True,
                by_alias=by_alias,
                include=include,
                exclude=exclude,
                exclude_unset=exclude_unset,
                exclude_defaults=exclude_defaults,
                exclude_none=exclude_none,
            )
        )



import json
import allure
from apicase.report.html.gen_report import gen_html_report
import requests
from typing import List
from apicase.common.logger import log
from apicase.common.schema import JSONAssertSchema
from apicase.common.exception import JsonAssertError


class AllureAttach(object):

    @classmethod
    def record_http(cls, response, validators: List[JSONAssertSchema]
                    , description: str = None, path: str = None):
        """

        :param path:
        :param description:
        :param response:
        :param validators:
        :return:
        """

        validate_extractor = []

        for i in validators:
            k = i.copy()
            k.comparator = k.comparator.__name__
            k = k.dict()
            k['check'] = k['check'].value
            validate_extractor.append(k)
        title_name = "Request.{0}: {1} {2}".format(response.request.method, path, description)
        content_size = int(dict(response.headers).get("content-length") or 0)
        summary = {
            'name': description,
            'path': response.request.url,
            'request': {
                'url': response.request.url,
                'headers': cls.headers_format(response.request.headers),
                'body': response.request.body
            },
            'response': {
                'status_code': response.status_code,
                'headers': cls.headers_format(response.headers),
                'body': response.text
            },
            'validators': {
                'validate_extractor': validate_extractor,
                'validate_script': [

                ],
            },
            'time': {

            },
            'stat': {
                'content_size': content_size
            }
        }
        body = gen_html_report(summary=summary)
        # log.debug(summary)
        allure.attach(body=body, name=title_name, attachment_type=allure.attachment_type.HTML)
        # 记录后，抛出断言错误

    @staticmethod
    def headers_format(headers: dict):
        """
        headers字典转为文本并进行编码处理
        :param headers:
        :return:
        """
        header_str = ''
        for i, k in headers.items():
            header_str += i + ': ' + k + '\n'
        return requests.utils.unquote(header_str)

    # @classmethod
    # def api_record(cls, response):
    #     name = "{0} {1}".format(response.request.method, response.request.url)
    #     req_headers = json.dumps(dict(response.request.headers), ensure_ascii=False)
    #     res_headers = json.dumps(dict(response.headers), ensure_ascii=False)
    #     req_body = response.request.body
    #     if type(req_body) == bytes:
    #         try:
    #             req_body = req_body.decode()
    #         except UnicodeDecodeError:
    #             req_body = 'form-data参数不做打印'
    #     data_template = {
    #         'request': {
    #             'body': req_body,
    #             'headers': req_headers,
    #         },
    #         'response': {
    #             'status_code': response.status_code or None,
    #             'body': response.text,
    #             'headers': res_headers,
    #             # 'cookies': response.cookies or None,
    #             'encoding': response.encoding,
    #             'reason': response.reason,
    #             'ok': response.ok,
    #             'response_time_s': response.elapsed.total_seconds(),
    #         }
    #     }
    #     data_template = json.dumps(data_template, indent=1, separators=(',          ', ': '), ensure_ascii=False)
    #     allure.attach(body=data_template, name=name, attachment_type=allure.attachment_type.JSON)
    #
    # @classmethod
    # def sql_record(cls, sql_info: dict):
    #     crud = sql_info['crud']
    #     url = sql_info['   url']
    #     name = None
    #     for i in crud:
    #         if crud[i]:
    #             name = str(i) + ' ' + str(url)
    #             break
    #     data = str(sql_info['sql']) + '\n' + str(sql_info['parameters'])
    #     allure.attach(body=data, name=name, attachment_type=allure.attachment_type.TEXT)

    # @classmethod
    # def schema_assert_record(cls, assert_json: dict, schema: dict) -> None:
    #     """
    #     TODO(GuoWenHe): 待扩展数组模式
    #     """
    #     assert_json = json.dumps(assert_json, ensure_ascii=False)
    #     schema = json.dumps(schema, ensure_ascii=False)
    #     name = 'Assert api_json->schema'
    #     data = 'assert_data:\n' + str(assert_json) + '\nassert_schema\n' + str(schema)
    #     allure.attach(body=data, name=name, attachment_type=allure.attachment_type.TEXT)

    # @classmethod
    # def diff_record(cls, json_data) -> None:
    #     name = 'Assert Diff'
    #     data_template = json.dumps(json_data, indent=1, separators=(',  ', ': '), ensure_ascii=False)
    #     allure.attach(body=data_template, name=name, attachment_type=allure.attachment_type.JSON)



"""
HttpRunner html report

- result: define resultclass for unittest TextTestRunner
- gen_report: render html report with jinja2 template

"""

# from httprunner.report.html.result import HtmlTestResult
# from httprunner.report.html.gen_report import gen_html_report
#
# __all__ = [
#     "HtmlTestResult",
#     "gen_html_report"
# ]